@@ -4,13 +4,14 @@ if [[ -z $CIRCLE_PULL_REQUEST ]]; then
     CACHEDIR=`pwd`/theano_cache
     rm -rf $CACHEDIR
     export THEANO_FLAGS=base_compiledir=$CACHEDIR
+    export AESARA_FLAGS=base_compiledir=$CACHEDIR
 
     git branch -D executed-notebooks || true
     git checkout -b executed-notebooks
 
     . venv/bin/activate
     cd docs
-    python run_notebooks.py $*
+    python run_notebooks.py
 
     git add examples/*.ipynb
     git -c user.name='circleci' -c user.email='circleci' commit -m "now with executed tutorials"
