@@ -4,25 +4,27 @@
 import glob
 import os
 import sys
-
-# Third-party
-import nbformat
-from nbconvert.preprocessors import CellExecutionError, ExecutePreprocessor
+import logging
 
 
 def process_notebook(filename, kernel_name=None):
+    import nbformat
+    from nbconvert.preprocessors import CellExecutionError, ExecutePreprocessor
+
     path = os.path.join(
         os.path.abspath("theano_cache"), "p{0}".format(os.getpid())
     )
     os.makedirs(path, exist_ok=True)
     os.environ["THEANO_FLAGS"] = "base_compiledir={0}".format(path)
+    os.environ["AESARA_FLAGS"] = os.environ["THEANO_FLAGS"]
 
     with open(filename) as f:
         notebook = nbformat.read(f, as_version=4)
 
     ep = ExecutePreprocessor(timeout=-1, kernel_name=kernel_name)
+    ep.log.setLevel(logging.DEBUG)
+    ep.log.addHandler(logging.StreamHandler())
 
-    print("running: {0}".format(filename))
     try:
         ep.preprocess(notebook, {"metadata": {"path": "examples/"}})
     except CellExecutionError as e:
@@ -43,5 +45,5 @@ if __name__ == '__main__':
 
     nbsphinx_kernel_name = os.environ.get('NBSPHINX_KERNEL', 'python3')
 
-    for filename in glob.glob(pattern):
+    for filename in sorted(glob.glob(pattern)):
         process_notebook(filename, kernel_name=nbsphinx_kernel_name)
