@@ -1,5 +1,6 @@
 # Standard library
 import os
+import warnings
 
 # Third-party
 import numpy as np
@@ -7,13 +8,13 @@ import numpy as np
 # Project
 from .logging import logger
 from .data_helpers import validate_prepare_data
-from .samples_helpers import is_P_unimodal
+from .samples_analysis import MAP_sample, is_P_unimodal
 from .likelihood_helpers import get_trend_design_matrix
 from .prior_helpers import validate_n_offsets, validate_poly_trend
 from .prior import JokerPrior, _validate_model
 from .src.fast_likelihood import CJokerHelper
-from .utils import tempfile_decorator
 from .samples import JokerSamples
+from .utils import DEFAULT_RNG, NUMPY_LT_1_17
 
 __all__ = ['TheJoker']
 
@@ -29,8 +30,8 @@ class TheJoker:
         parameters used in The Joker.
     pool : `schwimmbad.BasePool` (optional)
         A processing pool (default is a `schwimmbad.SerialPool` instance).
-    random_state : `numpy.random.RandomState` (optional)
-        A `numpy.random.RandomState` instance for controlling random number
+    random_state : `numpy.random.Generator` (optional)
+        A `numpy.random.Generator` instance for controlling random number
         generation.
     tempfile_path : str (optional)
         A location on disk where The Joker may store some temporary files. Any
@@ -39,7 +40,8 @@ class TheJoker:
         Default: ``~/.thejoker``
     """
 
-    def __init__(self, prior, pool=None, random_state=None, tempfile_path=None):
+    def __init__(self, prior, pool=None, random_state=None,
+                 tempfile_path=None):
 
         # set the processing pool
         if pool is None:
@@ -53,10 +55,25 @@ class TheJoker:
         # Set the parent random state - child processes get different states
         # based on the parent
         if random_state is None:
-            random_state = np.random.RandomState()
-        elif not isinstance(random_state, np.random.RandomState):
-            raise TypeError("Random state object must be a numpy RandomState "
-                            "instance, not '{0}'".format(type(random_state)))
+            random_state = DEFAULT_RNG()
+        elif (isinstance(random_state, np.random.RandomState) and
+                not NUMPY_LT_1_17):
+            warnings.warn("With thejoker>=v1.2, use numpy.random.Generator "
+                          "objects instead of RandomState objects to control "
+                          "random numbers.", DeprecationWarning)
+            tmp = np.random.Generator(np.random.MT19937())
+            tmp.bit_generator.state = random_state.get_state()
+            random_state = tmp
+        elif (not NUMPY_LT_1_17 and
+                not isinstance(random_state, np.random.Generator)):
+            raise TypeError("Random state object must be a "
+                            "numpy.random.Generator instance, not "
+                            f"'{type(random_state)}'")
+        elif (NUMPY_LT_1_17 and
+                not isinstance(random_state, np.random.RandomState)):
+            raise TypeError("Random state object must be a "
+                            "numpy.random.RandomState instance, not "
+                            f"'{type(random_state)}'")
         self.random_state = random_state
 
         # check if a JokerParams instance was passed in to specify the state
@@ -84,7 +101,7 @@ class TheJoker:
 
     def marginal_ln_likelihood(self, data, prior_samples, n_batches=None,
                                in_memory=False):
-        f"""
+        """
         Compute the marginal log-likelihood at each of the input prior samples.
 
         Parameters
@@ -134,10 +151,11 @@ class TheJoker:
                          max_posterior_samples=None,
                          n_linear_samples=1,
                          return_logprobs=False,
+                         return_all_logprobs=False,
                          n_batches=None,
                          randomize_prior_order=False,
                          in_memory=False):
-        f"""
+        """
         Run The Joker's rejection sampling on prior samples to get posterior
         samples for the input data.
 
@@ -170,6 +188,9 @@ class TheJoker:
         return_logprobs : bool (optional)
             Also return the log-prior and (marginal) log-likelihood values
             evaluated at each sample.
+        return_all_logprobs : bool (optional)
+            This will return the marginal log-likelihood values at every prior
+            sample used in this sampling! Use at your own (memory) risk.
         n_batches : int (optional)
             The number of batches to split the prior samples into before
             distributing for computation. If using the (default) serial
@@ -220,7 +241,8 @@ class TheJoker:
                 random_state=self.random_state,
                 ln_prior=ln_prior,
                 max_posterior_samples=max_posterior_samples,
-                n_linear_samples=n_linear_samples)
+                n_linear_samples=n_linear_samples,
+                return_all_logprobs=return_all_logprobs)
 
         else:
             samples = rejection_sample_helper(
@@ -233,7 +255,8 @@ class TheJoker:
                 n_linear_samples=n_linear_samples,
                 return_logprobs=return_logprobs,
                 n_batches=n_batches,
-                randomize_prior_order=randomize_prior_order)
+                randomize_prior_order=randomize_prior_order,
+                return_all_logprobs=return_all_logprobs)
 
         return samples
 
@@ -345,7 +368,7 @@ class TheJoker:
 
         return samples
 
-    def setup_mcmc(self, data, joker_samples, model=None, use_cached=False):
+    def setup_mcmc(self, data, joker_samples, model=None, custom_func=None):
         """
         Setup the model to run MCMC using pymc3.
 
@@ -362,6 +385,7 @@ class TheJoker:
         model : `pymc3.Model`
             This is either required, or this function must be called within a
             pymc3 model context.
+        custom_func : callable (optional)
 
         Returns
         -------
@@ -371,7 +395,7 @@ class TheJoker:
         import pymc3 as pm
         import exoplanet as xo
         import exoplanet.units as xu
-        import theano.tensor as tt
+        import aesara_theano_fallback.tensor as tt
 
         model = _validate_model(model)
 
@@ -379,7 +403,7 @@ class TheJoker:
         data, ids, _ = validate_prepare_data(data,
                                              self.prior.poly_trend,
                                              self.prior.n_offsets)
-        x = data._t_bmjd - data._t0_bmjd
+        x = data._t_bmjd - data._t_ref_bmjd
         y = data.rv.value
         err = data.rv_err.to_value(data.rv.unit)
 
@@ -393,41 +417,36 @@ class TheJoker:
             if not is_P_unimodal(joker_samples, data):
                 logger.warn("TODO: samples ain't unimodal")
 
-            joker_samples = joker_samples.median()
+            MAP_sample = joker_samples.median()
+
+        else:
+            MAP_sample = joker_samples
 
         mcmc_init = dict()
         for name in self.prior.par_names:
             unit = getattr(self.prior.pars[name], xu.UNIT_ATTR_NAME)
-            if joker_samples[name].shape == ():
-                mcmc_init[name] = joker_samples[name].to_value(unit)
-            else:
-                mcmc_init[name] = joker_samples[name].to_value(unit)[0]
-
-        if 't_peri' in model.named_vars and use_cached:
-            logger.debug('pymc3 model has already been setup for running MCMC '
-                         '- using the previously setup model parameters.')
-            return mcmc_init
-
-        # remove previously-added parameters:
-        names = ['t_peri', 'obs']
-        to_remove = []
-        for name in names:
-            for i, par in enumerate(model.vars):
-                if par.name == name:
-                    to_remove.append(par)
-                    del model.named_vars[name]
-        [model.vars.remove(p) for p in to_remove]
+            mcmc_init[name] = MAP_sample[name].to_value(unit)
+        if custom_func is not None:
+            mcmc_init = custom_func(mcmc_init, MAP_sample, model)
+        mcmc_init = {k: np.squeeze(v) for k, v in mcmc_init.items()}
 
         p = self.prior.pars
-        with model:
-            t_peri = pm.Deterministic('t_peri',
-                                      p['P'] * p['M0'] / (2*np.pi))
 
+        if 't_peri' not in model.named_vars:
+            with model:
+                pm.Deterministic('t_peri', p['P'] * p['M0'] / (2*np.pi))
+
+        if 'obs' in model.named_vars:
+            return mcmc_init
+
+        with model:
             # Set up the orbit model
-            orbit = xo.orbits.KeplerianOrbit(period=p['P'],
-                                             ecc=p['e'],
-                                             omega=p['omega'],
-                                             t_periastron=t_peri)
+            orbit = xo.orbits.KeplerianOrbit(
+                period=p['P'],
+                ecc=p['e'],
+                omega=p['omega'],
+                t_periastron=model.named_vars['t_peri']
+            )
 
         # design matrix
         M = get_trend_design_matrix(data, ids, self.prior.poly_trend)
@@ -444,10 +463,22 @@ class TheJoker:
             trend = tt.dot(M, v_trend_vec)
 
             rv_model = orbit.get_radial_velocity(x, K=p['K']) + trend
+            pm.Deterministic("model_rv", rv_model)
 
             err = tt.sqrt(err**2 + p['s']**2)
             pm.Normal("obs", mu=rv_model, sd=err, observed=y)
 
+            pm.Deterministic('logp', model.logpt)
+
+            dist = pm.Normal.dist(model.model_rv, data.rv_err.value)
+            lnlike = pm.Deterministic(
+                'ln_likelihood',
+                dist.logp(data.rv.value).sum(axis=-1))
+
+            pm.Deterministic(
+                'ln_prior',
+                model.logpt - lnlike)
+
         return mcmc_init
 
     def trace_to_samples(self, trace, data, names=None):
@@ -458,25 +489,11 @@ class TheJoker:
         ----------
         trace : `~pymc3.backends.base.MultiTrace`
         """
-        import pymc3 as pm
-        import exoplanet.units as xu
+        warnings.warn(
+            'This method is deprecated: Use '
+            'thejoker.samples_helpers.trace_to_samples() instead',
+            UserWarning
+        )
 
-        df = pm.trace_to_dataframe(trace)
-
-        data, *_ = validate_prepare_data(data,
-                                         self.prior.poly_trend,
-                                         self.prior.n_offsets)
-
-        samples = JokerSamples(poly_trend=self.prior.poly_trend,
-                               n_offsets=self.prior.n_offsets,
-                               t0=data.t0)
-
-        if names is None:
-            names = self.prior.par_names
-
-        for name in names:
-            par = self.prior.pars[name]
-            unit = getattr(par, xu.UNIT_ATTR_NAME)
-            samples[name] = df[name].values * unit
-
-        return samples
+        from thejoker.samples_helpers import trace_to_samples
+        return trace_to_samples(self, trace, data, names)
